@@ -617,6 +617,10 @@ class Pattern(object):
 
         return self.oscar(return_apgcode=True, eventual_oscillator=False, verbose=False)['apgcode']
 
+    def download_samples(self, **kwargs):
+
+        return self.owner.download_samples(self.getrule(), self.apgcode, **kwargs)
+
     @property
     def period(self):
 
