@@ -1,5 +1,6 @@
 from .lowlevel import *
 from .pattern import Pattern
+from .samples import download_samples as dsamples
 import os
 
 floats = {}
@@ -44,12 +45,14 @@ class Lifetree(object):
 
         return Pattern(self.session, pptr, self)
 
-    def viewer(self, filename=None, width=480, height=480,
-                lv_config='#C [[ THEME 6 GRID GRIDMAJOR 0 ]]'):
+    def hashsoup(self, rule, symmetry, seed):
 
-        if filename is None:
-            filename = self.session.newfloat('viewer') + '.html'
+        pptr = self.lifelib('Hashsoup', self.ptr, rule, symmetry, seed)
+        return Pattern(self.session, pptr, self)
 
+    def download_samples(self, rule, apgcode, **kwargs):
+
+        return dsamples(apgcode, rule, self, **kwargs)
 
     def pattern(self, rle="", rule=None, tempfile='tempfile', verify_apgcode=True):
 
